[% BLOCK page_title %]
  [% IF page_title %]
    <h1>[% page_title %]</h1>
  [% END %]
[% END %]

[% BLOCK logo %]
  <p><img src="/pix/Apache-JAF.gif" width="100" height="20" alt="Apache::JAF" border="0"></p>
[% END %]

