<!--// Apache::JAF sample site //-->
<html>
<head>
<title>[% page_title %][% ' / ' IF page_title %]Apache::JAF</title>
<link rel="stylesheet" type="text/css" href="/site.css">
<script language="JavaScript" src="/site.js"></script>
</head>

<body bgcolor="white">

[% PROCESS page_title %]
